<?php

/*
 Plugin Name: WP Sheet Editor - EDD Downloads
 Description: Edit Easy Digital Downloads in spreadsheet.
 Version:     1.0.6
 Author:      WP Sheet Editor
 Author URI:  https://wpsheeteditor.com
 Plugin URI: https://wpsheeteditor.com/go/edd-downloads-addon?utm_source=wp-admin&utm_medium=plugins-list&utm_campaign=edd
 License:     GPL2
 License URI: https://www.gnu.org/licenses/gpl-2.0.html
 @fs_premium_only /modules/user-path/send-user-path.php, /modules/advanced-filters/, /modules/columns-renaming/, /modules/formulas/, /modules/custom-columns/, /modules/spreadsheet-setup/, /modules/acf/, /modules/posts-templates/, /inc/integrations/
*/
require_once 'vendor/vg-plugin-sdk/index.php';
require_once 'vendor/freemius/start.php';
require_once 'inc/freemius-init.php';
if ( !class_exists( 'WP_Sheet_Editor_EDD_Downloads' ) ) {
    /**
     * Filter rows in the spreadsheet editor.
     */
    class WP_Sheet_Editor_EDD_Downloads
    {
        private static  $instance = false ;
        var  $plugin_url = null ;
        var  $plugin_dir = null ;
        var  $textname = 'vg_sheet_editor_edd_downloads' ;
        var  $buy_link = null ;
        var  $version = '1.0.6' ;
        var  $settings = null ;
        var  $args = null ;
        var  $vg_plugin_sdk = null ;
        var  $post_type = 'download' ;
        private function __construct()
        {
        }
        
        function init_plugin_sdk()
        {
            $this->args = array(
                'main_plugin_file'         => __FILE__,
                'show_welcome_page'        => true,
                'welcome_page_file'        => $this->plugin_dir . '/views/welcome-page-content.php',
                'logo'                     => plugins_url( '/assets/imgs/logo-248x102.png', __FILE__ ),
                'buy_link'                 => $this->buy_link,
                'plugin_name'              => 'Bulk Edit EDD Downloads',
                'plugin_prefix'            => 'wpseedd_',
                'show_whatsnew_page'       => true,
                'whatsnew_pages_directory' => $this->plugin_dir . '/views/whats-new/',
                'plugin_version'           => $this->version,
                'plugin_options'           => $this->settings,
            );
            $this->vg_plugin_sdk = new VG_Freemium_Plugin_SDK( $this->args );
        }
        
        function notify_wrong_core_version()
        {
            $plugin_data = get_plugin_data( __FILE__, false, false );
            ?>
			<div class="notice notice-error">
				<p><?php 
            _e( 'Please update the WP Sheet Editor plugin and all its extensions to the latest version, the CORE plugin should be version 2.5.3 or higher. The plugin "' . $plugin_data['Name'] . '" requires that version.', VGSE()->textname );
            ?></p>
			</div>
			<?php 
        }
        
        function init()
        {
            require_once __DIR__ . '/modules/init.php';
            $this->modules_controller = new WP_Sheet_Editor_CORE_Modules_Init( __DIR__ );
            $integrations = array_merge( glob( __DIR__ . '/inc/*.php' ), glob( __DIR__ . '/inc/integrations/*.php' ), glob( __DIR__ . '/inc/providers/*.php' ) );
            foreach ( $integrations as $integration_file ) {
                require_once $integration_file;
            }
            $this->plugin_url = plugins_url( '/', __FILE__ );
            $this->plugin_dir = __DIR__;
            $this->buy_link = wpseedd_fs()->checkout_url();
            $this->init_plugin_sdk();
            // After core has initialized
            add_action( 'vg_sheet_editor/initialized', array( $this, 'after_core_init' ) );
        }
        
        function after_core_init()
        {
            
            if ( version_compare( VGSE()->version, '2.5.3' ) < 0 ) {
                $this->notify_wrong_core_version();
                return;
            }
            
            // Override core buy link with this plugin´s
            VGSE()->buy_link = $this->buy_link;
            load_plugin_textdomain( $this->textname, false, basename( dirname( __FILE__ ) ) . '/lang/' );
            // Enable admin pages in case "frontend sheets" addon disabled them
            add_filter( 'vg_sheet_editor/register_admin_pages', '__return_true', 11 );
        }
        
        /**
         * Creates or returns an instance of this class.
         */
        static function get_instance()
        {
            
            if ( null == WP_Sheet_Editor_EDD_Downloads::$instance ) {
                WP_Sheet_Editor_EDD_Downloads::$instance = new WP_Sheet_Editor_EDD_Downloads();
                WP_Sheet_Editor_EDD_Downloads::$instance->init();
            }
            
            return WP_Sheet_Editor_EDD_Downloads::$instance;
        }
        
        function __set( $name, $value )
        {
            $this->{$name} = $value;
        }
        
        function __get( $name )
        {
            return $this->{$name};
        }
    
    }
}

if ( !function_exists( 'vgse_edd_downloads' ) ) {
    function vgse_edd_downloads()
    {
        return WP_Sheet_Editor_EDD_Downloads::get_instance();
    }
    
    vgse_edd_downloads();
}
