<?php

if ( !class_exists( 'WPSE_EDD_Downloads_Sheet' ) ) {
    class WPSE_EDD_Downloads_Sheet extends WPSE_Sheet_Factory
    {
        function __construct()
        {
            $allowed_columns = array();
            $allowed_columns = array(
                'ID',
                'post_title',
                'post_content',
                'post_status',
                'post_name',
                'view_post',
                '_thumbnail_id',
                'edd_price',
                'edd_download_files_file',
                'edd_download_files_name'
            );
            parent::__construct( array(
                'fs_object'          => wpseedd_fs(),
                'post_type'          => array( 'download' ),
                'post_type_label'    => array( 'EDD Downloads' ),
                'serialized_columns' => array(),
                'columns'            => array( $this, 'get_columns' ),
                'allowed_columns'    => $allowed_columns,
                'remove_columns'     => array(),
            ) );
            add_filter(
                'vg_sheet_editor/provider/post/update_item_meta',
                array( $this, 'filter_cell_data_for_saving' ),
                10,
                3
            );
            add_filter(
                'vg_sheet_editor/provider/post/get_item_meta',
                array( $this, 'filter_cell_data_for_readings' ),
                10,
                5
            );
            add_filter(
                'vg_sheet_editor/serialized_addon/column_settings',
                array( $this, 'filter_serialized_column_settings' ),
                10,
                5
            );
            add_filter(
                'vg_sheet_editor/columns/blacklisted_columns',
                array( $this, 'blacklist_index_column' ),
                10,
                2
            );
            add_filter( 'redux/options/' . VGSE()->options_key . '/sections', array( $this, 'add_settings_page_options' ) );
        }
        
        function blacklist_index_column( $columns, $provider )
        {
            
            if ( in_array( $provider, $this->post_type ) ) {
                for ( $i = 0 ;  $i < 20 ;  $i++ ) {
                    $columns[] = 'edd_variable_prices_index_i_' . $i;
                }
                $columns[] = "edd_variable_prices_name_i_0";
                $columns[] = "edd_variable_prices_amount_i_0";
                $columns[] = "edd_variable_prices_sale_price_i_0";
            }
            
            return $columns;
        }
        
        /**
         * Add fields to options page
         * @param array $sections
         * @return array
         */
        function add_settings_page_options( $sections )
        {
            $fields = array( array(
                'id'       => 'edd_max_files',
                'type'     => 'text',
                'validate' => 'numeric',
                'title'    => __( 'How many files do you add to the products?', VGSE()->textname ),
                'desc'     => __( 'We show 3 columns for every file so you can enter the file name, url, and required price. This is helpful to display only the necessary columns', VGSE()->textname ),
                'default'  => 3,
            ) );
            $sections[] = array(
                'icon'   => 'el-icon-cogs',
                'title'  => __( 'EDD sheet', VGSE()->textname ),
                'fields' => $fields,
            );
            return $sections;
        }
        
        function filter_serialized_column_settings(
            $settings,
            $first_set_keys,
            $field,
            $key,
            $post_type
        )
        {
            if ( !in_array( $post_type, $this->post_type ) ) {
                return $settings;
            }
            
            if ( strpos( $key, 'edd_download_files_condition' ) !== false ) {
                $settings['title'] = str_replace( 'Condition', __( 'Required price', VGSE()->textname ), $settings['title'] );
                $settings['default_value'] = 'all';
            }
            
            return $settings;
        }
        
        function filter_cell_data_for_readings(
            $value,
            $id,
            $key,
            $single,
            $context
        )
        {
            if ( $context !== 'read' || !in_array( get_post_type( $id ), $this->post_type ) ) {
                return $value;
            }
            return $value;
        }
        
        function filter_cell_data_for_saving( $new_value, $id, $key )
        {
            if ( !in_array( get_post_type( $id ), $this->post_type ) ) {
                return $new_value;
            }
            if ( $key === 'edd_download_files' && is_array( $new_value ) ) {
                foreach ( $new_value as $index => $file ) {
                    
                    if ( empty($file['condition']) ) {
                        $new_value[$index]['condition'] = 'all';
                        $file['condition'] = 'all';
                    }
                    
                    $variable_prices = VGSE()->helpers->get_current_provider()->get_item_meta( $id, 'edd_variable_prices', true );
                    if ( !is_numeric( $file['condition'] ) && $file['condition'] !== 'all' && !empty($variable_prices) && ($price = wp_list_filter( $variable_prices, array(
                        'name' => $file['condition'],
                    ) )) ) {
                        $new_value[$index]['condition'] = $price[0]['index'];
                    }
                }
            }
            return $new_value;
        }
        
        function get_columns()
        {
            $settings = get_option( VGSE()->options_key, array() );
            $columns = array();
            $columns['_edd_download_sales'] = array(
                'data_type'     => 'meta_data',
                'title'         => __( 'Sales', vgse_edd_downloads()->textname ),
                'allow_to_save' => false,
            );
            $columns['_edd_download_earnings'] = array(
                'data_type'     => 'meta_data',
                'title'         => __( 'Earnings', vgse_edd_downloads()->textname ),
                'allow_to_save' => false,
            );
            $serialized_fields = array();
            $serialized_fields[] = array(
                'sample_field_key'               => 'edd_variable_prices',
                'sample_field'                   => array( array(
                'name'   => '',
                'amount' => '',
            ) ),
                'column_width'                   => 175,
                'column_title_prefix'            => __( 'Variable Price', vgse_edd_downloads()->textname ),
                'level'                          => ( !empty($settings['edd_max_variable_prices']) ? (int) $settings['edd_max_variable_prices'] : 3 ),
                'allowed_post_types'             => array( vgse_edd_downloads()->post_type ),
                'is_single_level'                => false,
                'allow_in_wc_product_variations' => false,
                'index_start'                    => 1,
                'label_index_start'              => 1,
            );
            $serialized_fields[] = array(
                'sample_field_key'               => 'edd_download_files',
                'sample_field'                   => array( array(
                'file'      => '',
                'name'      => '',
                'condition' => '',
            ) ),
                'column_width'                   => 150,
                'column_title_prefix'            => __( 'Files', vgse_edd_downloads()->textname ),
                'level'                          => ( !empty($settings['edd_max_files']) ? (int) $settings['edd_max_files'] : 3 ),
                'allowed_post_types'             => array( vgse_edd_downloads()->post_type ),
                'is_single_level'                => false,
                'allow_in_wc_product_variations' => false,
                'index_start'                    => 1,
                'label_index_start'              => 1,
            );
            $serialized_fields[] = array(
                'sample_field_key'               => '_edd_bundled_products',
                'sample_field'                   => array( '' ),
                'column_width'                   => 150,
                'column_title_prefix'            => __( 'Bundle product', vgse_edd_downloads()->textname ),
                'level'                          => ( !empty($settings['edd_max_bundle_products']) ? (int) $settings['edd_max_bundle_products'] : 3 ),
                'allowed_post_types'             => array( vgse_edd_downloads()->post_type ),
                'is_single_level'                => true,
                'allow_in_wc_product_variations' => false,
            );
            $serialized_fields[] = array(
                'sample_field_key'               => '_edd_bundled_products_conditions',
                'sample_field'                   => array( '' ),
                'column_width'                   => 150,
                'column_title_prefix'            => __( 'Bundle required price', vgse_edd_downloads()->textname ),
                'level'                          => ( !empty($settings['edd_max_bundle_products']) ? (int) $settings['edd_max_bundle_products'] : 3 ),
                'allowed_post_types'             => array( vgse_edd_downloads()->post_type ),
                'is_single_level'                => true,
                'allow_in_wc_product_variations' => false,
            );
            foreach ( $serialized_fields as $serialized_field ) {
                new WP_Sheet_Editor_Serialized_Field( $serialized_field );
            }
            return $columns;
        }
    
    }
    new WPSE_EDD_Downloads_Sheet();
}
